@@ -16,13 +16,13 @@
 """If the title is empty, try to extract track and title from the
 filename.
 """
-from __future__ import (division, absolute_import, print_function,
-                        unicode_literals)
+from __future__ import division, absolute_import, print_function
 
 from beets import plugins
 from beets.util import displayable_path
 import os
 import re
+import six
 
 
 # Filename field extraction patterns.
@@ -101,7 +101,7 @@ def apply_matches(d):
     """Given a mapping from items to field dicts, apply the fields to
     the objects.
     """
-    some_map = d.values()[0]
+    some_map = list(d.values())[0]
     keys = some_map.keys()
 
     # Only proceed if the "tag" field is equal across all filenames.
@@ -133,7 +133,7 @@ def apply_matches(d):
     # Apply the title and track.
     for item in d:
         if bad_title(item.title):
-            item.title = unicode(d[item][title_field])
+            item.title = six.text_type(d[item][title_field])
         if 'track' in d[item] and item.track == 0:
             item.track = int(d[item]['track'])
 
