@@ -52,14 +52,14 @@ class KeyFinderPlugin(BeetsPlugin):
 
     def find_key(self, items, write=False):
         overwrite = self.config['overwrite'].get(bool)
-        bin = util.bytestring_path(self.config['bin'].get(unicode))
+        bin = self.config['bin'].as_str()
 
         for item in items:
             if item['initial_key'] and not overwrite:
                 continue
 
             try:
-                output = util.command_output([bin, b'-f',
+                output = util.command_output([bin, '-f',
                                               util.syspath(item.path)])
             except (subprocess.CalledProcessError, OSError) as exc:
                 self._log.error(u'execution failed: {0}', exc)
@@ -73,7 +73,7 @@ class KeyFinderPlugin(BeetsPlugin):
 
             key_raw = output.rsplit(None, 1)[-1]
             try:
-                key = key_raw.decode('utf8')
+                key = util.text_string(key_raw)
             except UnicodeDecodeError:
                 self._log.error(u'output is invalid UTF-8')
                 continue
