@@ -27,6 +27,7 @@ from beets.plugins import BeetsPlugin
 import os
 import socket
 from beets import config
+import six
 
 
 # No need to introduce a dependency on an MPD library for such a
@@ -67,7 +68,7 @@ class MPDUpdatePlugin(BeetsPlugin):
     def __init__(self):
         super(MPDUpdatePlugin, self).__init__()
         config['mpd'].add({
-            'host':     u'localhost',
+            'host':     os.environ.get('MPD_HOST', u'localhost'),
             'port':     6600,
             'password': u'',
         })
@@ -86,9 +87,9 @@ class MPDUpdatePlugin(BeetsPlugin):
 
     def update(self, lib):
         self.update_mpd(
-            config['mpd']['host'].get(unicode),
+            config['mpd']['host'].as_str(),
             config['mpd']['port'].get(int),
-            config['mpd']['password'].get(unicode),
+            config['mpd']['password'].as_str(),
         )
 
     def update_mpd(self, host='localhost', port=6600, password=None):
@@ -101,7 +102,7 @@ class MPDUpdatePlugin(BeetsPlugin):
             s = BufferedSocket(host, port)
         except socket.error as e:
             self._log.warning(u'MPD connection failed: {0}',
-                              unicode(e.strerror))
+                              six.text_type(e.strerror))
             return
 
         resp = s.readline()
